@@ -1,6 +1,6 @@
 package App::SimpleScan::Plugin::LinkCheck;
 
-$VERSION = '0.02';
+$VERSION = '1.00';
 
 use warnings;
 use strict;
@@ -50,7 +50,8 @@ sub _do_has_link {
   my($self, $args) = @_;
   my($name, $compare, $count);
   if (!defined $args) {
-    $self->stack_test( qq(fail "No arguments for %%has_link";\n) );
+    $self->stack_code( qq(fail "No arguments for %%has_link";\n) );
+    $self->test_count( $self->test_count() + 1 );
     return;
   }
   else {
@@ -64,7 +65,8 @@ sub _do_has_link {
 sub _do_no_link {
   my($self, $args) = @_;
   if (!defined $args) {
-    $self->stack_test( qq(fail "No arguments for %%no_link";\n) );
+    $self->stack_code( qq(fail "No arguments for %%no_link";\n) );
+    $self->test_count( $self->test_count() + 1 );
   }
   else {
     my ($name) = $self->_extract_quotelike_args($args);
@@ -82,11 +84,13 @@ sub _add_link_condition {
   push @{ $self->{Link_conditions}->{ $condition->{name} } }, $condition;
 }
 
-sub per_test {
-  my($class, $testspec) = @_;
-  my $self = $testspec->app;
+sub filters {
+  return \&filter;
+}
+
+sub filter {
+  my($self, @code) = @_;
   return unless defined $self->_link_conditions;
-  my @code;
   my $test_count = 0;
 
   for my $link_name (keys %{$self->_link_conditions()} ) {
@@ -132,7 +136,8 @@ sub per_test {
       }
     }
   }
-  return $test_count, @code;
+  $self->test_count($self->test_count() + $test_count);
+  return @code;
 }
 
 
@@ -182,30 +187,96 @@ __END__
 
 =head1 NAME
 
-App::SimpleScan::Plugin::LinkCheck - [One line description of module's purpose here]
+App::SimpleScan::Plugin::LinkCheck - Link counting/presence/absence plugin 
 
 
 =head1 VERSION
 
-This document describes App::SimpleScan::Plugin::LinkCheck version 0.0.1
+This document describes App::SimpleScan::Plugin::LinkCheck version 1.00
 
 
 =head1 SYNOPSIS
 
-    use App::SimpleScan::Plugin::LinkCheck;
+    # After this plugin is installed:
+    %%has_link 'My stuff'
+    %%no_link  'Send email'
+
+    # The has_link and no_link checks will run for every test spec.
+    http://someplace.com /foo/ Y Got my page
+    http://someplace.com/subpage /foo/ Y Got my subpage
+
+    # Stop checking for the 'My stuff' link
+    %%forget_link 'My stuff'
+
+    # Now only the "no 'Send email' link' test will run.
+    http://someplace.com/map  /foo/ Y Get map page
+
+    # More specific tests can be done:
+    # The page will have exactly one 'home' link
+    %%has_link 'Home' == 1
+
+    # The page will have a non-zero number of 'offsite' links.
+    %%has_link 'offsite' > 0
+
+    http://someplace.com/links /foo/ Y Get links page
+    # Now both these tests and the 'send email' test will run.
 
-=for author to fill in:
-    Brief code example(s) here showing commonest usage(s).
-    This section will be as far as many users bother reading
-    so make it as educational and exeplary as possible.
-  
   
 =head1 DESCRIPTION
 
-=for author to fill in:
-    Write a full description of the module and its features here.
-    Use subsections (=head2, =head3) as appropriate.
+Th Linkcheck plugin adds simple link counting and presence/absence
+checking pragmas to C<simple_scan>. These pragmas are assertions;
+they will be run for every test spec occurring after the pragma(s)
+appear in the intput file.
 
+You can, for instance, assert that you do not want to see a link
+named 'Click here' anywhere in the pages referenced by the test
+specs in this test:
+
+  %%no_link 'Click here' 
+
+Additional test specs generated by variable substitution will
+each have the check made, so for example
+
+  %%which bar baz quux
+  http://<which>.foo.com  /foo.com/ Y   Fetch page
+
+would check for /foo.com/ on each of bar.foo.com, 
+baz.foo.com, and quux.foo.com, and also check for zero links
+whose name is 'Click here' on each of these pages.
+
+Conversely, you can assert that a link must be present on
+each page you look at:
+
+  %%has_link 'back'
+
+This means that a link named 'back' must appear on every page
+you fetch after this pragma occurs in the input.
+
+You may want to remove the condition for some pages in your
+tests; this can easily be done by 'forgetting' the link:
+
+  %%forget_link 'back'
+
+The assertion has now been dropped, and will not be tested
+through the rest of the file. If you want to forget all of
+the currently-active link-count assertions, just use
+
+  %%forget_all_links
+
+and all of the link checks will stop.
+
+If you want to be more specific about the link counts, 
+you can use the extended syntax for '%%has_link':
+
+  %%has_link 'beta' >= 0
+  %%has_link 'up' == 2
+  %%has_link 'report_problems' != 1
+
+Note that you can use any of the I<numeric> operators and
+a count for this. These work just like the other tests, in
+that they apply to every test spec following until either
+the end of the file, or a '%%forget_link' that drops them.
 
 =head1 INTERFACE 
 
@@ -217,9 +288,14 @@ Sets up the initial (empty) link conditions.
 
 Exports the definitions of C<has_link> and C<no_link> to C<simple_scan>.
 
-=head2 per_test
+=head2 filters
 
-Emits code to test all of the active link conditions for each testspec.
+Returns list of output filter subs to caller (standard callback).
+
+=head2 filter
+
+Determines what (if any) link checks are queued up and stackes them,
+incrementing the test count appropriately.
 
 =head1 DIAGNOSTICS
 
@@ -239,50 +315,21 @@ see the link was not something that looks like a number to Perl.
 
 =head1 CONFIGURATION AND ENVIRONMENT
 
-=for author to fill in:
-    A full explanation of any configuration system(s) used by the
-    module, including the names and locations of any configuration
-    files, and the meaning of any environment variables or properties
-    that can be set. These descriptions must also include details of any
-    configuration language used.
-  
 App::SimpleScan::Plugin::LinkCheck requires no configuration files or environment variables.
 
 
 =head1 DEPENDENCIES
 
-=for author to fill in:
-    A list of all the other modules that this module relies upon,
-    including any restrictions on versions, and an indication whether
-    the module is part of the standard Perl distribution, part of the
-    module's distribution, or must be installed separately. ]
-
-None.
+App::SimpleScan.
 
 
 =head1 INCOMPATIBILITIES
 
-=for author to fill in:
-    A list of any modules that this module cannot be used in conjunction
-    with. This may be due to name conflicts in the interface, or
-    competition for system or program resources, or due to internal
-    limitations of Perl (for example, many modules that use source code
-    filters are mutually incompatible).
-
 None reported.
 
 
 =head1 BUGS AND LIMITATIONS
 
-=for author to fill in:
-    A list of known problems with the module, together with some
-    indication Whether they are likely to be fixed in an upcoming
-    release. Also a list of restrictions on the features the module
-    does provide: data types that cannot be handled, performance issues
-    and the circumstances in which they may arise, practical
-    limitations on the size of data sets, special cases that are not
-    (yet) handled, etc.
-
 No bugs have been reported.
 
 Please report any bugs or feature requests to
@@ -297,7 +344,8 @@ Joe McMahon  C<< <mcmahon@yahoo-inc.com> >>
 
 =head1 LICENCE AND COPYRIGHT
 
-Copyright (c) 2006, Joe McMahon C<< <mcmahon@yahoo-inc.com> >>. All rights reserved.
+Copyright (c) 2006, Yahoo! and 
+Joe McMahon C<< <mcmahon@yahoo-inc.com> >>. All rights reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself. See L<perlartistic>.
